# ********************************************************************
#
#  $Id: helloworld.py 32630 2018-10-10 14:11:07Z seb $
#
#  An example that show how to use a  Yocto-3D
#
#  You can find more information on our web site:
#   Yocto-3D documentation:
#      https://www.yoctopuce.com/EN/products/yocto-3d/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
#
# *********************************************************************

#!/usr/bin/python
# -*- coding: utf-8 -*-
import os, sys
# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))

from yocto_api import *
from yocto_tilt import *
from yocto_compass import *
from yocto_gyro import *
from yocto_accelerometer import *


def usage():
    scriptname = os.path.basename(sys.argv[0])
    print("Usage:")
    print(scriptname + ' <serial_number>')
    print(scriptname + ' <logical_name>')
    print(scriptname + ' any  ')
    sys.exit()


def die(msg):
    sys.exit(msg + ' (check USB cable)')


errmsg = YRefParam()

if len(sys.argv) < 2:
    usage()

target = sys.argv[1]

# Setup the API to use local USB devices
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

if target == 'any':
    # retreive any tilt sensor
    anytilt = YTilt.FirstTilt()
    if anytilt is None:
        die('No module connected (check USB cable)')
    m = anytilt.get_module()
    target = m.get_serialNumber()
else:
    anytilt = YTilt.FindTilt(target + ".tilt1")
    if not (anytilt.isOnline()):
        die('Module not connected (check identification and USB cable)')

serial = anytilt.get_module().get_serialNumber()
tilt1 = YTilt.FindTilt(serial + ".tilt1")
tilt2 = YTilt.FindTilt(serial + ".tilt2")
compass = YCompass.FindCompass(serial + ".compass")
accelerometer = YAccelerometer.FindAccelerometer(serial + ".accelerometer")
gyro = YGyro.FindGyro(serial + ".gyro")

count = 0

if not (tilt1.isOnline()):
    die("Module not connected (check identification and USB cable)")

while tilt1.isOnline():

    if count % 10 == 0:
        print("tilt1   tilt2   compass acc     gyro")

    print("%-7.1f " % tilt1.get_currentValue() + \
          "%-7.1f " % tilt2.get_currentValue() + \
          "%-7.1f " % compass.get_currentValue() + \
          "%-7.1f " % accelerometer.get_currentValue() + \
          "%-7.1f" % gyro.get_currentValue())
    count += 1
    YAPI.Sleep(250, errmsg)
YAPI.FreeAPI()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# add ../../Sources to the PYTHONPATH
import sys
import os

sys.path.append(os.path.join("..", "..", "Sources"))
from yocto_api import *
from yocto_colorled import *


def main():
    errmsg = YRefParam()
    # Setup the API to use local USB devices
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        sys.exit("init error" + errmsg.value)

    led = YColorLed.FirstColorLed()
    if led is None:
        sys.exit("No led connected (check USB cable)")

    led.resetBlinkSeq()  # cleans the sequence
    led.addRgbMoveToBlinkSeq(0x00FF00, 500)  # move to green in 500 ms
    led.addRgbMoveToBlinkSeq(0x000000, 0)  # switch to black instantaneously
    led.addRgbMoveToBlinkSeq(0x000000, 250)  # stays black for 250ms
    led.addRgbMoveToBlinkSeq(0x0000FF, 0)  # switch to blue instantaneously
    led.addRgbMoveToBlinkSeq(0x0000FF, 100)  # stays blue for 100ms
    led.addRgbMoveToBlinkSeq(0x000000, 0)  # switch to black instantaneously
    led.addRgbMoveToBlinkSeq(0x000000, 250)  # stays black for 250ms
    led.addRgbMoveToBlinkSeq(0xFF0000, 0)  # switch to red instantaneously
    led.addRgbMoveToBlinkSeq(0xFF0000, 100)  # stays red for 100ms
    led.addRgbMoveToBlinkSeq(0x000000, 0)  # switch to black instantaneously
    led.addRgbMoveToBlinkSeq(0x000000, 1000)  # stays black for 1s
    led.startBlinkSeq()  # starts sequence
    print("The led is now blinking autonomously")
    YAPI.FreeAPI()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_cellular.py 66072 2025-04-30 06:59:12Z mvuilleu $
#
#  Implements the asyncio YCellular API for Cellular functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YCellular
version: PATCH_WITH_VERSION
requires: yocto_api
requires: yocto_cellular_aio
"""
from __future__ import annotations
import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import const, _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True
    _DYNAMIC_HELPERS: Final[bool] = True

from .yocto_cellular_aio import (
    YCellular as YCellular_aio,
    YCellRecord
)
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (generated code: YCellular class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YCellularValueCallback = Union[Callable[['YCellular', str], Awaitable[None]], None]
    except TypeError:
        YCellularValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YCellular(YFunction):
    """
    The YCellular class provides control over cellular network parameters
    and status for devices that are GSM-enabled.
    Note that TCP/IP parameters are configured separately, using class YNetwork.

    """
    _aio: YCellular_aio
    # --- (end of generated code: YCellular class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YCellular return codes)
        LINKQUALITY_INVALID: Final[int] = YAPI.INVALID_UINT
        CELLOPERATOR_INVALID: Final[str] = YAPI.INVALID_STRING
        CELLIDENTIFIER_INVALID: Final[str] = YAPI.INVALID_STRING
        IMSI_INVALID: Final[str] = YAPI.INVALID_STRING
        MESSAGE_INVALID: Final[str] = YAPI.INVALID_STRING
        PIN_INVALID: Final[str] = YAPI.INVALID_STRING
        RADIOCONFIG_INVALID: Final[str] = YAPI.INVALID_STRING
        LOCKEDOPERATOR_INVALID: Final[str] = YAPI.INVALID_STRING
        APN_INVALID: Final[str] = YAPI.INVALID_STRING
        APNSECRET_INVALID: Final[str] = YAPI.INVALID_STRING
        PINGINTERVAL_INVALID: Final[int] = YAPI.INVALID_UINT
        DATASENT_INVALID: Final[int] = YAPI.INVALID_UINT
        DATARECEIVED_INVALID: Final[int] = YAPI.INVALID_UINT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        CELLTYPE_GPRS: Final[int] = 0
        CELLTYPE_EGPRS: Final[int] = 1
        CELLTYPE_WCDMA: Final[int] = 2
        CELLTYPE_HSDPA: Final[int] = 3
        CELLTYPE_NONE: Final[int] = 4
        CELLTYPE_CDMA: Final[int] = 5
        CELLTYPE_LTE_M: Final[int] = 6
        CELLTYPE_NB_IOT: Final[int] = 7
        CELLTYPE_EC_GSM_IOT: Final[int] = 8
        CELLTYPE_INVALID: Final[int] = -1
        AIRPLANEMODE_OFF: Final[int] = 0
        AIRPLANEMODE_ON: Final[int] = 1
        AIRPLANEMODE_INVALID: Final[int] = -1
        ENABLEDATA_HOMENETWORK: Final[int] = 0
        ENABLEDATA_ROAMING: Final[int] = 1
        ENABLEDATA_NEVER: Final[int] = 2
        ENABLEDATA_NEUTRALITY: Final[int] = 3
        ENABLEDATA_INVALID: Final[int] = -1
        # --- (end of generated code: YCellular return codes)

    # --- (generated code: YCellular attributes declaration)
    _linkQuality: int
    _cellOperator: str
    _cellIdentifier: str
    _cellType: int
    _imsi: str
    _message: str
    _pin: str
    _radioConfig: str
    _lockedOperator: str
    _airplaneMode: int
    _enableData: int
    _apn: str
    _apnSecret: str
    _pingInterval: int
    _dataSent: int
    _dataReceived: int
    _command: str
    _valueCallbackCellular: YCellularValueCallback
    # --- (end of generated code: YCellular attributes declaration)

    # --- (generated code: YCellular implementation)

    @classmethod
    def FirstCellular(cls) -> Union[YCellular, None]:
        """
        Starts the enumeration of cellular interfaces currently accessible.
        Use the method YCellular.nextCellular() to iterate on
        next cellular interfaces.

        @return a pointer to a YCellular object, corresponding to
                the first cellular interface currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YCellular_aio.FirstCellular())

    @classmethod
    def FirstCellularInContext(cls, yctx: YAPIContext) -> Union[YCellular, None]:
        """
        Starts the enumeration of cellular interfaces currently accessible.
        Use the method YCellular.nextCellular() to iterate on
        next cellular interfaces.

        @param yctx : a YAPI context.

        @return a pointer to a YCellular object, corresponding to
                the first cellular interface currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YCellular_aio.FirstCellularInContext(yctx))

    def nextCellular(self):
        """
        Continues the enumeration of cellular interfaces started using yFirstCellular().
        Caution: You can't make any assumption about the returned cellular interfaces order.
        If you want to find a specific a cellular interface, use Cellular.findCellular()
        and a hardwareID or a logical name.

        @return a pointer to a YCellular object, corresponding to
                a cellular interface currently online, or a None pointer
                if there are no more cellular interfaces to enumerate.
        """
        return self._proxy(type(self), self._aio.nextCellular())

    if not _DYNAMIC_HELPERS:
        def get_linkQuality(self) -> int:
            """
            Returns the link quality, expressed in percent.

            @return an integer corresponding to the link quality, expressed in percent

            On failure, throws an exception or returns YCellular.LINKQUALITY_INVALID.
            """
            return self._run(self._aio.get_linkQuality())

    if not _DYNAMIC_HELPERS:
        def get_cellOperator(self) -> str:
            """
            Returns the name of the cell operator currently in use.

            @return a string corresponding to the name of the cell operator currently in use

            On failure, throws an exception or returns YCellular.CELLOPERATOR_INVALID.
            """
            return self._run(self._aio.get_cellOperator())

    if not _DYNAMIC_HELPERS:
        def get_cellIdentifier(self) -> str:
            """
            Returns the unique identifier of the cellular antenna in use: MCC, MNC, LAC and Cell ID.

            @return a string corresponding to the unique identifier of the cellular antenna in use: MCC, MNC,
            LAC and Cell ID

            On failure, throws an exception or returns YCellular.CELLIDENTIFIER_INVALID.
            """
            return self._run(self._aio.get_cellIdentifier())

    if not _DYNAMIC_HELPERS:
        def get_cellType(self) -> int:
            """
            Active cellular connection type.

            @return a value among YCellular.CELLTYPE_GPRS, YCellular.CELLTYPE_EGPRS, YCellular.CELLTYPE_WCDMA,
            YCellular.CELLTYPE_HSDPA, YCellular.CELLTYPE_NONE, YCellular.CELLTYPE_CDMA,
            YCellular.CELLTYPE_LTE_M, YCellular.CELLTYPE_NB_IOT and YCellular.CELLTYPE_EC_GSM_IOT

            On failure, throws an exception or returns YCellular.CELLTYPE_INVALID.
            """
            return self._run(self._aio.get_cellType())

    if not _DYNAMIC_HELPERS:
        def get_imsi(self) -> str:
            """
            Returns the International Mobile Subscriber Identity (MSI) that uniquely identifies
            the SIM card. The first 3 digits represent the mobile country code (MCC), which
            is followed by the mobile network code (MNC), either 2-digit (European standard)
            or 3-digit (North American standard)

            @return a string corresponding to the International Mobile Subscriber Identity (MSI) that uniquely identifies
                    the SIM card

            On failure, throws an exception or returns YCellular.IMSI_INVALID.
            """
            return self._run(self._aio.get_imsi())

    if not _DYNAMIC_HELPERS:
        def get_message(self) -> str:
            """
            Returns the latest status message from the wireless interface.

            @return a string corresponding to the latest status message from the wireless interface

            On failure, throws an exception or returns YCellular.MESSAGE_INVALID.
            """
            return self._run(self._aio.get_message())

    if not _DYNAMIC_HELPERS:
        def get_pin(self) -> str:
            """
            Returns an opaque string if a PIN code has been configured in the device to access
            the SIM card, or an empty string if none has been configured or if the code provided
            was rejected by the SIM card.

            @return a string corresponding to an opaque string if a PIN code has been configured in the device to access
                    the SIM card, or an empty string if none has been configured or if the code provided
                    was rejected by the SIM card

            On failure, throws an exception or returns YCellular.PIN_INVALID.
            """
            return self._run(self._aio.get_pin())

    if not _DYNAMIC_HELPERS:
        def set_pin(self, newval: str) -> int:
            """
            Changes the PIN code used by the module to access the SIM card.
            This function does not change the code on the SIM card itself, but only changes
            the parameter used by the device to try to get access to it. If the SIM code
            does not work immediately on first try, it will be automatically forgotten
            and the message will be set to "Enter SIM PIN". The method should then be
            invoked again with right correct PIN code. After three failed attempts in a row,
            the message is changed to "Enter SIM PUK" and the SIM card PUK code must be
            provided using method sendPUK.

            Remember to call the saveToFlash() method of the module to save the
            new value in the device flash.

            @param newval : a string corresponding to the PIN code used by the module to access the SIM card

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_pin(newval))

    if not _DYNAMIC_HELPERS:
        def get_radioConfig(self) -> str:
            """
            Returns the type of protocol used over the serial line, as a string.
            Possible values are "Line" for ASCII messages separated by CR and/or LF,
            "Frame:[timeout]ms" for binary messages separated by a delay time,
            "Char" for a continuous ASCII stream or
            "Byte" for a continuous binary stream.

            @return a string corresponding to the type of protocol used over the serial line, as a string

            On failure, throws an exception or returns YCellular.RADIOCONFIG_INVALID.
            """
            return self._run(self._aio.get_radioConfig())

    if not _DYNAMIC_HELPERS:
        def set_radioConfig(self, newval: str) -> int:
            """
            Changes the type of protocol used over the serial line.
            Possible values are "Line" for ASCII messages separated by CR and/or LF,
            "Frame:[timeout]ms" for binary messages separated by a delay time,
            "Char" for a continuous ASCII stream or
            "Byte" for a continuous binary stream.
            The suffix "/[wait]ms" can be added to reduce the transmit rate so that there
            is always at lest the specified number of milliseconds between each bytes sent.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the type of protocol used over the serial line

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_radioConfig(newval))

    if not _DYNAMIC_HELPERS:
        def get_lockedOperator(self) -> str:
            """
            Returns the name of the only cell operator to use if automatic choice is disabled,
            or an empty string if the SIM card will automatically choose among available
            cell operators.

            @return a string corresponding to the name of the only cell operator to use if automatic choice is disabled,
                    or an empty string if the SIM card will automatically choose among available
                    cell operators

            On failure, throws an exception or returns YCellular.LOCKEDOPERATOR_INVALID.
            """
            return self._run(self._aio.get_lockedOperator())

    if not _DYNAMIC_HELPERS:
        def set_lockedOperator(self, newval: str) -> int:
            """
            Changes the name of the cell operator to be used. If the name is an empty
            string, the choice will be made automatically based on the SIM card. Otherwise,
            the selected operator is the only one that will be used.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a string corresponding to the name of the cell operator to be used

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_lockedOperator(newval))

    if not _DYNAMIC_HELPERS:
        def get_airplaneMode(self) -> int:
            """
            Returns true if the airplane mode is active (radio turned off).

            @return either YCellular.AIRPLANEMODE_OFF or YCellular.AIRPLANEMODE_ON, according to true if the
            airplane mode is active (radio turned off)

            On failure, throws an exception or returns YCellular.AIRPLANEMODE_INVALID.
            """
            return self._run(self._aio.get_airplaneMode())

    if not _DYNAMIC_HELPERS:
        def set_airplaneMode(self, newval: int) -> int:
            """
            Changes the activation state of airplane mode (radio turned off).

            @param newval : either YCellular.AIRPLANEMODE_OFF or YCellular.AIRPLANEMODE_ON, according to the
            activation state of airplane mode (radio turned off)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_airplaneMode(newval))

    if not _DYNAMIC_HELPERS:
        def get_enableData(self) -> int:
            """
            Returns the condition for enabling IP data services (GPRS).
            When data services are disabled, SMS are the only mean of communication.

            @return a value among YCellular.ENABLEDATA_HOMENETWORK, YCellular.ENABLEDATA_ROAMING,
            YCellular.ENABLEDATA_NEVER and YCellular.ENABLEDATA_NEUTRALITY corresponding to the condition for
            enabling IP data services (GPRS)

            On failure, throws an exception or returns YCellular.ENABLEDATA_INVALID.
            """
            return self._run(self._aio.get_enableData())

    if not _DYNAMIC_HELPERS:
        def set_enableData(self, newval: int) -> int:
            """
            Changes the condition for enabling IP data services (GPRS).
            The service can be either fully deactivated, or limited to the SIM home network,
            or enabled for all partner networks (roaming). Caution: enabling data services
            on roaming networks may cause prohibitive communication costs !

            When data services are disabled, SMS are the only mean of communication.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a value among YCellular.ENABLEDATA_HOMENETWORK, YCellular.ENABLEDATA_ROAMING,
            YCellular.ENABLEDATA_NEVER and YCellular.ENABLEDATA_NEUTRALITY corresponding to the condition for
            enabling IP data services (GPRS)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_enableData(newval))

    if not _DYNAMIC_HELPERS:
        def get_apn(self) -> str:
            """
            Returns the Access Point Name (APN) to be used, if needed.
            When left blank, the APN suggested by the cell operator will be used.

            @return a string corresponding to the Access Point Name (APN) to be used, if needed

            On failure, throws an exception or returns YCellular.APN_INVALID.
            """
            return self._run(self._aio.get_apn())

    if not _DYNAMIC_HELPERS:
        def set_apn(self, newval: str) -> int:
            """
            Returns the Access Point Name (APN) to be used, if needed.
            When left blank, the APN suggested by the cell operator will be used.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a string

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_apn(newval))

    if not _DYNAMIC_HELPERS:
        def get_apnSecret(self) -> str:
            """
            Returns an opaque string if APN authentication parameters have been configured
            in the device, or an empty string otherwise.
            To configure these parameters, use set_apnAuth().

            @return a string corresponding to an opaque string if APN authentication parameters have been configured
                    in the device, or an empty string otherwise

            On failure, throws an exception or returns YCellular.APNSECRET_INVALID.
            """
            return self._run(self._aio.get_apnSecret())

    if not _DYNAMIC_HELPERS:
        def set_apnSecret(self, newval: str) -> int:
            return self._run(self._aio.set_apnSecret(newval))

    if not _DYNAMIC_HELPERS:
        def get_pingInterval(self) -> int:
            """
            Returns the automated connectivity check interval, in seconds.

            @return an integer corresponding to the automated connectivity check interval, in seconds

            On failure, throws an exception or returns YCellular.PINGINTERVAL_INVALID.
            """
            return self._run(self._aio.get_pingInterval())

    if not _DYNAMIC_HELPERS:
        def set_pingInterval(self, newval: int) -> int:
            """
            Changes the automated connectivity check interval, in seconds.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the automated connectivity check interval, in seconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_pingInterval(newval))

    if not _DYNAMIC_HELPERS:
        def get_dataSent(self) -> int:
            """
            Returns the number of bytes sent so far.

            @return an integer corresponding to the number of bytes sent so far

            On failure, throws an exception or returns YCellular.DATASENT_INVALID.
            """
            return self._run(self._aio.get_dataSent())

    if not _DYNAMIC_HELPERS:
        def set_dataSent(self, newval: int) -> int:
            """
            Changes the value of the outgoing data counter.

            @param newval : an integer corresponding to the value of the outgoing data counter

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_dataSent(newval))

    if not _DYNAMIC_HELPERS:
        def get_dataReceived(self) -> int:
            """
            Returns the number of bytes received so far.

            @return an integer corresponding to the number of bytes received so far

            On failure, throws an exception or returns YCellular.DATARECEIVED_INVALID.
            """
            return self._run(self._aio.get_dataReceived())

    if not _DYNAMIC_HELPERS:
        def set_dataReceived(self, newval: int) -> int:
            """
            Changes the value of the incoming data counter.

            @param newval : an integer corresponding to the value of the incoming data counter

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_dataReceived(newval))

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindCellular(cls, func: str) -> YCellular:
        """
        Retrieves a cellular interface for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the cellular interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YCellular.isOnline() to test if the cellular interface is
        indeed online at a given time. In case of ambiguity when looking for
        a cellular interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the cellular interface, for instance
                YHUBGSM1.cellular.

        @return a YCellular object allowing you to drive the cellular interface.
        """
        return cls._proxy(cls, YCellular_aio.FindCellular(func))

    @classmethod
    def FindCellularInContext(cls, yctx: YAPIContext, func: str) -> YCellular:
        """
        Retrieves a cellular interface for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the cellular interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YCellular.isOnline() to test if the cellular interface is
        indeed online at a given time. In case of ambiguity when looking for
        a cellular interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the cellular interface, for instance
                YHUBGSM1.cellular.

        @return a YCellular object allowing you to drive the cellular interface.
        """
        return cls._proxy(cls, YCellular_aio.FindCellularInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YCellularValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def sendPUK(self, puk: str, newPin: str) -> int:
            """
            Sends a PUK code to unlock the SIM card after three failed PIN code attempts, and
            set up a new PIN into the SIM card. Only ten consecutive tentatives are permitted:
            after that, the SIM card will be blocked permanently without any mean of recovery
            to use it again. Note that after calling this method, you have usually to invoke
            method set_pin() to tell the YoctoHub which PIN to use in the future.

            @param puk : the SIM PUK code
            @param newPin : new PIN code to configure into the SIM card

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.sendPUK(puk, newPin))

    if not _DYNAMIC_HELPERS:
        def set_apnAuth(self, username: str, password: str) -> int:
            """
            Configure authentication parameters to connect to the APN. Both
            PAP and CHAP authentication are supported.

            @param username : APN username
            @param password : APN password

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_apnAuth(username, password))

    if not _DYNAMIC_HELPERS:
        def clearDataCounters(self) -> int:
            """
            Clear the transmitted data counters.

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.clearDataCounters())

    if not _DYNAMIC_HELPERS:
        def _AT(self, cmd: str) -> str:
            """
            Sends an AT command to the GSM module and returns the command output.
            The command will only execute when the GSM module is in standard
            command state, and should leave it in the exact same state.
            Use this function with great care !

            @param cmd : the AT command to execute, like for instance: "+CCLK?".

            @return a string with the result of the commands. Empty lines are
                    automatically removed from the output.
            """
            return self._run(self._aio._AT(cmd))

    if not _DYNAMIC_HELPERS:
        def get_availableOperators(self) -> list[str]:
            """
            Returns the list detected cell operators in the neighborhood.
            This function will typically take between 30 seconds to 1 minute to
            return. Note that any SIM card can usually only connect to specific
            operators. All networks returned by this function might therefore
            not be available for connection.

            @return a list of string (cell operator names).
            """
            return self._run(self._aio.get_availableOperators())

    if not _DYNAMIC_HELPERS:
        def quickCellSurvey(self) -> list[YCellRecord]:
            """
            Returns a list of nearby cellular antennas, as required for quick
            geolocation of the device. The first cell listed is the serving
            cell, and the next ones are the neighbor cells reported by the
            serving cell.

            @return a list of YCellRecords.
            """
            return self._run(self._aio.quickCellSurvey())

    if not _DYNAMIC_HELPERS:
        def decodePLMN(self, mccmnc: str) -> str:
            """
            Returns the cell operator brand for a given MCC/MNC pair (DEPRECATED).

            @param mccmnc : a string starting with a MCC code followed by a MNC code,

            @return a string containing the corresponding cell operator brand name.
            """
            return self._run(self._aio.decodePLMN(mccmnc))

    if not _DYNAMIC_HELPERS:
        def get_communicationProfiles(self) -> list[str]:
            """
            Returns the list available radio communication profiles, as a string array
            (YoctoHub-GSM-4G only).
            Each string is a made of a numerical ID, followed by a colon,
            followed by the profile description.

            @return a list of string describing available radio communication profiles.
            """
            return self._run(self._aio.get_communicationProfiles())

    # --- (end of generated code: YCellular implementation)




# ********************************************************************
#
#  $Id: helloworld.py 32628 2018-10-10 13:37:59Z seb $
#
#  An example that show how to use a  Yocto-RangeFinder
#
#  You can find more information on our web site:
#   Yocto-RangeFinder documentation:
#      https://www.yoctopuce.com/EN/products/yocto-rangefinder/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
#
# *********************************************************************

#!/usr/bin/python
# -*- coding: utf-8 -*-
import os, sys
# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))

from yocto_api import *
from yocto_rangefinder import *
from yocto_lightsensor import *
from yocto_temperature import *

def usage():
    scriptname = os.path.basename(sys.argv[0])
    print("Usage:")
    print(scriptname + ' <serial_number>')
    print(scriptname + ' <logical_name>')
    print(scriptname + ' any  ')
    sys.exit()

def die(msg):
    sys.exit(msg + ' (check USB cable)')

errmsg = YRefParam()
if len(sys.argv) < 2:
    usage()
target = sys.argv[1]

# Setup the API to use local USB devices
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

if target == 'any':
    # retrieve any Range finder
    rf = YRangeFinder.FirstRangeFinder()
    if rf is None:
        die('No module connected')
    target = rf.get_module().get_serialNumber()
else:
    rf = YRangeFinder.FindRangeFinder(target + '.rangeFinder1')

if not (rf.isOnline()):
    die('device not connected')

ir  = YLightSensor.FindLightSensor(target+'.lightSensor1')
tmp = YTemperature.FindTemperature(target+'.temperature1')

while rf.isOnline():
    print("Distance    :  " + str(int(rf.get_currentValue())) )
    print("Ambiant IR  :  " + str(int(ir.get_currentValue())) )
    print("Temperature :  " + str(int(tmp.get_currentValue())) )
    YAPI.Sleep(1000)

YAPI.FreeAPI()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YAnButton
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YAnButton
version: PATCH_WITH_VERSION
requires: yocto_anbutton_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_anbutton_aio import YAnButton as YAnButton_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YAnButton class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YAnButtonValueCallback = Union[Callable[['YAnButton', str], Awaitable[None]], None]
    except TypeError:
        YAnButtonValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YAnButton(YFunction):
    """
    The YAnButton class provide access to basic resistive inputs.
    Such inputs can be used to measure the state
    of a simple button as well as to read an analog potentiometer (variable resistance).
    This can be use for instance with a continuous rotating knob, a throttle grip
    or a joystick. The module is capable to calibrate itself on min and max values,
    in order to compute a calibrated value that varies proportionally with the
    potentiometer position, regardless of its total resistance.

    """
    _aio: YAnButton_aio
    # --- (end of YAnButton class start)
    if not _IS_MICROPYTHON:
        # --- (YAnButton return codes)
        CALIBRATEDVALUE_INVALID: Final[int] = YAPI.INVALID_UINT
        RAWVALUE_INVALID: Final[int] = YAPI.INVALID_UINT
        CALIBRATIONMAX_INVALID: Final[int] = YAPI.INVALID_UINT
        CALIBRATIONMIN_INVALID: Final[int] = YAPI.INVALID_UINT
        SENSITIVITY_INVALID: Final[int] = YAPI.INVALID_UINT
        LASTTIMEPRESSED_INVALID: Final[int] = YAPI.INVALID_LONG
        LASTTIMERELEASED_INVALID: Final[int] = YAPI.INVALID_LONG
        PULSECOUNTER_INVALID: Final[int] = YAPI.INVALID_LONG
        PULSETIMER_INVALID: Final[int] = YAPI.INVALID_LONG
        ANALOGCALIBRATION_OFF: Final[int] = 0
        ANALOGCALIBRATION_ON: Final[int] = 1
        ANALOGCALIBRATION_INVALID: Final[int] = -1
        ISPRESSED_FALSE: Final[int] = 0
        ISPRESSED_TRUE: Final[int] = 1
        ISPRESSED_INVALID: Final[int] = -1
        INPUTTYPE_ANALOG_FAST: Final[int] = 0
        INPUTTYPE_DIGITAL4: Final[int] = 1
        INPUTTYPE_ANALOG_SMOOTH: Final[int] = 2
        INPUTTYPE_DIGITAL_FAST: Final[int] = 3
        INPUTTYPE_INVALID: Final[int] = -1
        # --- (end of YAnButton return codes)


    # --- (YAnButton implementation)

    @classmethod
    def FirstAnButton(cls) -> Union[YAnButton, None]:
        """
        Starts the enumeration of analog inputs currently accessible.
        Use the method YAnButton.nextAnButton() to iterate on
        next analog inputs.

        @return a pointer to a YAnButton object, corresponding to
                the first analog input currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YAnButton_aio.FirstAnButton())

    @classmethod
    def FirstAnButtonInContext(cls, yctx: YAPIContext) -> Union[YAnButton, None]:
        """
        Starts the enumeration of analog inputs currently accessible.
        Use the method YAnButton.nextAnButton() to iterate on
        next analog inputs.

        @param yctx : a YAPI context.

        @return a pointer to a YAnButton object, corresponding to
                the first analog input currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YAnButton_aio.FirstAnButtonInContext(yctx))

    def nextAnButton(self):
        """
        Continues the enumeration of analog inputs started using yFirstAnButton().
        Caution: You can't make any assumption about the returned analog inputs order.
        If you want to find a specific an analog input, use AnButton.findAnButton()
        and a hardwareID or a logical name.

        @return a pointer to a YAnButton object, corresponding to
                an analog input currently online, or a None pointer
                if there are no more analog inputs to enumerate.
        """
        return self._proxy(type(self), self._aio.nextAnButton())

    if not _DYNAMIC_HELPERS:
        def get_calibratedValue(self) -> int:
            """
            Returns the current calibrated input value (between 0 and 1000, included).

            @return an integer corresponding to the current calibrated input value (between 0 and 1000, included)

            On failure, throws an exception or returns YAnButton.CALIBRATEDVALUE_INVALID.
            """
            return self._run(self._aio.get_calibratedValue())

    if not _DYNAMIC_HELPERS:
        def get_rawValue(self) -> int:
            """
            Returns the current measured input value as-is (between 0 and 4095, included).

            @return an integer corresponding to the current measured input value as-is (between 0 and 4095, included)

            On failure, throws an exception or returns YAnButton.RAWVALUE_INVALID.
            """
            return self._run(self._aio.get_rawValue())

    if not _DYNAMIC_HELPERS:
        def get_analogCalibration(self) -> int:
            """
            Tells if a calibration process is currently ongoing.

            @return either YAnButton.ANALOGCALIBRATION_OFF or YAnButton.ANALOGCALIBRATION_ON

            On failure, throws an exception or returns YAnButton.ANALOGCALIBRATION_INVALID.
            """
            return self._run(self._aio.get_analogCalibration())

    if not _DYNAMIC_HELPERS:
        def set_analogCalibration(self, newval: int) -> int:
            """
            Starts or stops the calibration process. Remember to call the saveToFlash()
            method of the module at the end of the calibration if the modification must be kept.

            @param newval : either YAnButton.ANALOGCALIBRATION_OFF or YAnButton.ANALOGCALIBRATION_ON

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_analogCalibration(newval))

    if not _DYNAMIC_HELPERS:
        def get_calibrationMax(self) -> int:
            """
            Returns the maximal value measured during the calibration (between 0 and 4095, included).

            @return an integer corresponding to the maximal value measured during the calibration (between 0
            and 4095, included)

            On failure, throws an exception or returns YAnButton.CALIBRATIONMAX_INVALID.
            """
            return self._run(self._aio.get_calibrationMax())

    if not _DYNAMIC_HELPERS:
        def set_calibrationMax(self, newval: int) -> int:
            """
            Changes the maximal calibration value for the input (between 0 and 4095, included), without actually
            starting the automated calibration.  Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the maximal calibration value for the input (between 0
            and 4095, included), without actually
                    starting the automated calibration

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_calibrationMax(newval))

    if not _DYNAMIC_HELPERS:
        def get_calibrationMin(self) -> int:
            """
            Returns the minimal value measured during the calibration (between 0 and 4095, included).

            @return an integer corresponding to the minimal value measured during the calibration (between 0
            and 4095, included)

            On failure, throws an exception or returns YAnButton.CALIBRATIONMIN_INVALID.
            """
            return self._run(self._aio.get_calibrationMin())

    if not _DYNAMIC_HELPERS:
        def set_calibrationMin(self, newval: int) -> int:
            """
            Changes the minimal calibration value for the input (between 0 and 4095, included), without actually
            starting the automated calibration.  Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the minimal calibration value for the input (between 0
            and 4095, included), without actually
                    starting the automated calibration

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_calibrationMin(newval))

    if not _DYNAMIC_HELPERS:
        def get_sensitivity(self) -> int:
            """
            Returns the sensibility for the input (between 1 and 1000) for triggering user callbacks.

            @return an integer corresponding to the sensibility for the input (between 1 and 1000) for
            triggering user callbacks

            On failure, throws an exception or returns YAnButton.SENSITIVITY_INVALID.
            """
            return self._run(self._aio.get_sensitivity())

    if not _DYNAMIC_HELPERS:
        def set_sensitivity(self, newval: int) -> int:
            """
            Changes the sensibility for the input (between 1 and 1000) for triggering user callbacks.
            The sensibility is used to filter variations around a fixed value, but does not preclude the
            transmission of events when the input value evolves constantly in the same direction.
            Special case: when the value 1000 is used, the callback will only be thrown when the logical state
            of the input switches from pressed to released and back.
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : an integer corresponding to the sensibility for the input (between 1 and 1000) for
            triggering user callbacks

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_sensitivity(newval))

    if not _DYNAMIC_HELPERS:
        def get_isPressed(self) -> int:
            """
            Returns true if the input (considered as binary) is active (closed contact), and false otherwise.

            @return either YAnButton.ISPRESSED_FALSE or YAnButton.ISPRESSED_TRUE, according to true if the
            input (considered as binary) is active (closed contact), and false otherwise

            On failure, throws an exception or returns YAnButton.ISPRESSED_INVALID.
            """
            return self._run(self._aio.get_isPressed())

    if not _DYNAMIC_HELPERS:
        def get_lastTimePressed(self) -> int:
            """
            Returns the number of elapsed milliseconds between the module power on and the last time
            the input button was pressed (the input contact transitioned from open to closed).

            @return an integer corresponding to the number of elapsed milliseconds between the module power on
            and the last time
                    the input button was pressed (the input contact transitioned from open to closed)

            On failure, throws an exception or returns YAnButton.LASTTIMEPRESSED_INVALID.
            """
            return self._run(self._aio.get_lastTimePressed())

    if not _DYNAMIC_HELPERS:
        def get_lastTimeReleased(self) -> int:
            """
            Returns the number of elapsed milliseconds between the module power on and the last time
            the input button was released (the input contact transitioned from closed to open).

            @return an integer corresponding to the number of elapsed milliseconds between the module power on
            and the last time
                    the input button was released (the input contact transitioned from closed to open)

            On failure, throws an exception or returns YAnButton.LASTTIMERELEASED_INVALID.
            """
            return self._run(self._aio.get_lastTimeReleased())

    if not _DYNAMIC_HELPERS:
        def get_pulseCounter(self) -> int:
            """
            Returns the pulse counter value. The value is a 32 bit integer. In case
            of overflow (>=2^32), the counter will wrap. To reset the counter, just
            call the resetCounter() method.

            @return an integer corresponding to the pulse counter value

            On failure, throws an exception or returns YAnButton.PULSECOUNTER_INVALID.
            """
            return self._run(self._aio.get_pulseCounter())

    if not _DYNAMIC_HELPERS:
        def set_pulseCounter(self, newval: int) -> int:
            return self._run(self._aio.set_pulseCounter(newval))

    if not _DYNAMIC_HELPERS:
        def get_pulseTimer(self) -> int:
            """
            Returns the timer of the pulses counter (ms).

            @return an integer corresponding to the timer of the pulses counter (ms)

            On failure, throws an exception or returns YAnButton.PULSETIMER_INVALID.
            """
            return self._run(self._aio.get_pulseTimer())

    if not _DYNAMIC_HELPERS:
        def get_inputType(self) -> int:
            """
            Returns the decoding method applied to the input (analog or multiplexed binary switches).

            @return a value among YAnButton.INPUTTYPE_ANALOG_FAST, YAnButton.INPUTTYPE_DIGITAL4,
            YAnButton.INPUTTYPE_ANALOG_SMOOTH and YAnButton.INPUTTYPE_DIGITAL_FAST corresponding to the
            decoding method applied to the input (analog or multiplexed binary switches)

            On failure, throws an exception or returns YAnButton.INPUTTYPE_INVALID.
            """
            return self._run(self._aio.get_inputType())

    if not _DYNAMIC_HELPERS:
        def set_inputType(self, newval: int) -> int:
            """
            Changes the decoding method applied to the input (analog or multiplexed binary switches).
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : a value among YAnButton.INPUTTYPE_ANALOG_FAST, YAnButton.INPUTTYPE_DIGITAL4,
            YAnButton.INPUTTYPE_ANALOG_SMOOTH and YAnButton.INPUTTYPE_DIGITAL_FAST corresponding to the
            decoding method applied to the input (analog or multiplexed binary switches)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_inputType(newval))

    @classmethod
    def FindAnButton(cls, func: str) -> YAnButton:
        """
        Retrieves an analog input for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the analog input is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YAnButton.isOnline() to test if the analog input is
        indeed online at a given time. In case of ambiguity when looking for
        an analog input by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the analog input, for instance
                YBUZZER2.anButton1.

        @return a YAnButton object allowing you to drive the analog input.
        """
        return cls._proxy(cls, YAnButton_aio.FindAnButton(func))

    @classmethod
    def FindAnButtonInContext(cls, yctx: YAPIContext, func: str) -> YAnButton:
        """
        Retrieves an analog input for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the analog input is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YAnButton.isOnline() to test if the analog input is
        indeed online at a given time. In case of ambiguity when looking for
        an analog input by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the analog input, for instance
                YBUZZER2.anButton1.

        @return a YAnButton object allowing you to drive the analog input.
        """
        return cls._proxy(cls, YAnButton_aio.FindAnButtonInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YAnButtonValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def resetCounter(self) -> int:
            """
            Returns the pulse counter value as well as its timer.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.resetCounter())

    # --- (end of YAnButton implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YServo
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YServo
version: PATCH_WITH_VERSION
requires: yocto_servo_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_servo_aio import YServo as YServo_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YServo class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YServoValueCallback = Union[Callable[['YServo', str], Awaitable[None]], None]
    except TypeError:
        YServoValueCallback = Union[Callable, Awaitable]
if _IS_MICROPYTHON:
    from collections import namedtuple
    YMove = namedtuple("YMove", ('target', 'ms', 'moving'))
else:
    from typing import NamedTuple
    class YMove(NamedTuple):
        target: int
        ms: int
        moving: int


# noinspection PyProtectedMember
class YServo(YFunction):
    """
    The YServo class is designed to drive remote-control servo motors
    outputs. This class allows you not only to move
    a servo to a given position, but also to specify the time interval
    in which the move should be performed. This makes it possible to
    synchronize two servos involved in a same move.

    """
    _aio: YServo_aio
    # --- (end of YServo class start)
    if not _IS_MICROPYTHON:
        # --- (YServo return codes)
        POSITION_INVALID: Final[int] = YAPI.INVALID_INT
        RANGE_INVALID: Final[int] = YAPI.INVALID_UINT
        NEUTRAL_INVALID: Final[int] = YAPI.INVALID_UINT
        MOVE_INVALID: Final[None] = None
        POSITIONATPOWERON_INVALID: Final[int] = YAPI.INVALID_INT
        ENABLED_FALSE: Final[int] = 0
        ENABLED_TRUE: Final[int] = 1
        ENABLED_INVALID: Final[int] = -1
        ENABLEDATPOWERON_FALSE: Final[int] = 0
        ENABLEDATPOWERON_TRUE: Final[int] = 1
        ENABLEDATPOWERON_INVALID: Final[int] = -1
        # --- (end of YServo return codes)


    # --- (YServo implementation)

    @classmethod
    def FirstServo(cls) -> Union[YServo, None]:
        """
        Starts the enumeration of RC servo motors currently accessible.
        Use the method YServo.nextServo() to iterate on
        next RC servo motors.

        @return a pointer to a YServo object, corresponding to
                the first RC servo motor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YServo_aio.FirstServo())

    @classmethod
    def FirstServoInContext(cls, yctx: YAPIContext) -> Union[YServo, None]:
        """
        Starts the enumeration of RC servo motors currently accessible.
        Use the method YServo.nextServo() to iterate on
        next RC servo motors.

        @param yctx : a YAPI context.

        @return a pointer to a YServo object, corresponding to
                the first RC servo motor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YServo_aio.FirstServoInContext(yctx))

    def nextServo(self):
        """
        Continues the enumeration of RC servo motors started using yFirstServo().
        Caution: You can't make any assumption about the returned RC servo motors order.
        If you want to find a specific a RC servo motor, use Servo.findServo()
        and a hardwareID or a logical name.

        @return a pointer to a YServo object, corresponding to
                a RC servo motor currently online, or a None pointer
                if there are no more RC servo motors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextServo())

    if not _DYNAMIC_HELPERS:
        def get_position(self) -> int:
            """
            Returns the current servo position.

            @return an integer corresponding to the current servo position

            On failure, throws an exception or returns YServo.POSITION_INVALID.
            """
            return self._run(self._aio.get_position())

    if not _DYNAMIC_HELPERS:
        def set_position(self, newval: int) -> int:
            """
            Changes immediately the servo driving position.

            @param newval : an integer corresponding to immediately the servo driving position

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_position(newval))

    if not _DYNAMIC_HELPERS:
        def get_enabled(self) -> int:
            """
            Returns the state of the RC servo motors.

            @return either YServo.ENABLED_FALSE or YServo.ENABLED_TRUE, according to the state of the RC servo motors

            On failure, throws an exception or returns YServo.ENABLED_INVALID.
            """
            return self._run(self._aio.get_enabled())

    if not _DYNAMIC_HELPERS:
        def set_enabled(self, newval: int) -> int:
            """
            Stops or starts the RC servo motor.

            @param newval : either YServo.ENABLED_FALSE or YServo.ENABLED_TRUE

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_enabled(newval))

    if not _DYNAMIC_HELPERS:
        def get_range(self) -> int:
            """
            Returns the current range of use of the servo.

            @return an integer corresponding to the current range of use of the servo

            On failure, throws an exception or returns YServo.RANGE_INVALID.
            """
            return self._run(self._aio.get_range())

    if not _DYNAMIC_HELPERS:
        def set_range(self, newval: int) -> int:
            """
            Changes the range of use of the servo, specified in per cents.
            A range of 100% corresponds to a standard control signal, that varies
            from 1 [ms] to 2 [ms], When using a servo that supports a double range,
            from 0.5 [ms] to 2.5 [ms], you can select a range of 200%.
            Be aware that using a range higher than what is supported by the servo
            is likely to damage the servo. Remember to call the matching module
            saveToFlash() method, otherwise this call will have no effect.

            @param newval : an integer corresponding to the range of use of the servo, specified in per cents

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_range(newval))

    if not _DYNAMIC_HELPERS:
        def get_neutral(self) -> int:
            """
            Returns the duration in microseconds of a neutral pulse for the servo.

            @return an integer corresponding to the duration in microseconds of a neutral pulse for the servo

            On failure, throws an exception or returns YServo.NEUTRAL_INVALID.
            """
            return self._run(self._aio.get_neutral())

    if not _DYNAMIC_HELPERS:
        def set_neutral(self, newval: int) -> int:
            """
            Changes the duration of the pulse corresponding to the neutral position of the servo.
            The duration is specified in microseconds, and the standard value is 1500 [us].
            This setting makes it possible to shift the range of use of the servo.
            Be aware that using a range higher than what is supported by the servo is
            likely to damage the servo. Remember to call the matching module
            saveToFlash() method, otherwise this call will have no effect.

            @param newval : an integer corresponding to the duration of the pulse corresponding to the neutral
            position of the servo

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_neutral(newval))

    if not _DYNAMIC_HELPERS:
        def set_move(self, newval: YMove) -> int:
            return self._run(self._aio.set_move(newval))

    if not _DYNAMIC_HELPERS:
        def move(self, target, ms_duration) -> int:
            """
            Performs a smooth move at constant speed toward a given position.

            @param target      : new position at the end of the move
            @param ms_duration : total duration of the move, in milliseconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.move(target, ms_duration))

    if not _DYNAMIC_HELPERS:
        def get_positionAtPowerOn(self) -> int:
            """
            Returns the servo position at device power up.

            @return an integer corresponding to the servo position at device power up

            On failure, throws an exception or returns YServo.POSITIONATPOWERON_INVALID.
            """
            return self._run(self._aio.get_positionAtPowerOn())

    if not _DYNAMIC_HELPERS:
        def set_positionAtPowerOn(self, newval: int) -> int:
            """
            Configure the servo position at device power up. Remember to call the matching
            module saveToFlash() method, otherwise this call will have no effect.

            @param newval : an integer

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_positionAtPowerOn(newval))

    if not _DYNAMIC_HELPERS:
        def get_enabledAtPowerOn(self) -> int:
            """
            Returns the servo signal generator state at power up.

            @return either YServo.ENABLEDATPOWERON_FALSE or YServo.ENABLEDATPOWERON_TRUE, according to the
            servo signal generator state at power up

            On failure, throws an exception or returns YServo.ENABLEDATPOWERON_INVALID.
            """
            return self._run(self._aio.get_enabledAtPowerOn())

    if not _DYNAMIC_HELPERS:
        def set_enabledAtPowerOn(self, newval: int) -> int:
            """
            Configure the servo signal generator state at power up. Remember to call the matching module saveToFlash()
            method, otherwise this call will have no effect.

            @param newval : either YServo.ENABLEDATPOWERON_FALSE or YServo.ENABLEDATPOWERON_TRUE

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_enabledAtPowerOn(newval))

    @classmethod
    def FindServo(cls, func: str) -> YServo:
        """
        Retrieves a RC servo motor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the RC servo motor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YServo.isOnline() to test if the RC servo motor is
        indeed online at a given time. In case of ambiguity when looking for
        a RC servo motor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the RC servo motor, for instance
                SERVORC1.servo1.

        @return a YServo object allowing you to drive the RC servo motor.
        """
        return cls._proxy(cls, YServo_aio.FindServo(func))

    @classmethod
    def FindServoInContext(cls, yctx: YAPIContext, func: str) -> YServo:
        """
        Retrieves a RC servo motor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the RC servo motor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YServo.isOnline() to test if the RC servo motor is
        indeed online at a given time. In case of ambiguity when looking for
        a RC servo motor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the RC servo motor, for instance
                SERVORC1.servo1.

        @return a YServo object allowing you to drive the RC servo motor.
        """
        return cls._proxy(cls, YServo_aio.FindServoInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YServoValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    # --- (end of YServo implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YGps
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YGps
version: PATCH_WITH_VERSION
requires: yocto_gps_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_gps_aio import YGps as YGps_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YGps class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YGpsValueCallback = Union[Callable[['YGps', str], Awaitable[None]], None]
    except TypeError:
        YGpsValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YGps(YFunction):
    """
    The YGps class allows you to retrieve positioning
    data from a GPS/GNSS sensor. This class can provides
    complete positioning information. However, if you
    wish to define callbacks on position changes or record
    the position in the datalogger, you
    should use the YLatitude et YLongitude classes.

    """
    _aio: YGps_aio
    # --- (end of YGps class start)
    if not _IS_MICROPYTHON:
        # --- (YGps return codes)
        SATCOUNT_INVALID: Final[int] = YAPI.INVALID_LONG
        SATPERCONST_INVALID: Final[int] = YAPI.INVALID_LONG
        GPSREFRESHRATE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        LATITUDE_INVALID: Final[str] = YAPI.INVALID_STRING
        LONGITUDE_INVALID: Final[str] = YAPI.INVALID_STRING
        DILUTION_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        ALTITUDE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        GROUNDSPEED_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        DIRECTION_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        UNIXTIME_INVALID: Final[int] = YAPI.INVALID_LONG
        DATETIME_INVALID: Final[str] = YAPI.INVALID_STRING
        UTCOFFSET_INVALID: Final[int] = YAPI.INVALID_INT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        ISFIXED_FALSE: Final[int] = 0
        ISFIXED_TRUE: Final[int] = 1
        ISFIXED_INVALID: Final[int] = -1
        COORDSYSTEM_GPS_DMS: Final[int] = 0
        COORDSYSTEM_GPS_DM: Final[int] = 1
        COORDSYSTEM_GPS_D: Final[int] = 2
        COORDSYSTEM_INVALID: Final[int] = -1
        CONSTELLATION_GNSS: Final[int] = 0
        CONSTELLATION_GPS: Final[int] = 1
        CONSTELLATION_GLONASS: Final[int] = 2
        CONSTELLATION_GALILEO: Final[int] = 3
        CONSTELLATION_GPS_GLONASS: Final[int] = 4
        CONSTELLATION_GPS_GALILEO: Final[int] = 5
        CONSTELLATION_GLONASS_GALILEO: Final[int] = 6
        CONSTELLATION_INVALID: Final[int] = -1
        # --- (end of YGps return codes)


    # --- (YGps implementation)

    @classmethod
    def FirstGps(cls) -> Union[YGps, None]:
        """
        Starts the enumeration of geolocalization modules currently accessible.
        Use the method YGps.nextGps() to iterate on
        next geolocalization modules.

        @return a pointer to a YGps object, corresponding to
                the first geolocalization module currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YGps_aio.FirstGps())

    @classmethod
    def FirstGpsInContext(cls, yctx: YAPIContext) -> Union[YGps, None]:
        """
        Starts the enumeration of geolocalization modules currently accessible.
        Use the method YGps.nextGps() to iterate on
        next geolocalization modules.

        @param yctx : a YAPI context.

        @return a pointer to a YGps object, corresponding to
                the first geolocalization module currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YGps_aio.FirstGpsInContext(yctx))

    def nextGps(self):
        """
        Continues the enumeration of geolocalization modules started using yFirstGps().
        Caution: You can't make any assumption about the returned geolocalization modules order.
        If you want to find a specific a geolocalization module, use Gps.findGps()
        and a hardwareID or a logical name.

        @return a pointer to a YGps object, corresponding to
                a geolocalization module currently online, or a None pointer
                if there are no more geolocalization modules to enumerate.
        """
        return self._proxy(type(self), self._aio.nextGps())

    if not _DYNAMIC_HELPERS:
        def get_isFixed(self) -> int:
            """
            Returns TRUE if the receiver has found enough satellites to work.

            @return either YGps.ISFIXED_FALSE or YGps.ISFIXED_TRUE, according to TRUE if the receiver has found
            enough satellites to work

            On failure, throws an exception or returns YGps.ISFIXED_INVALID.
            """
            return self._run(self._aio.get_isFixed())

    if not _DYNAMIC_HELPERS:
        def get_satCount(self) -> int:
            """
            Returns the total count of satellites used to compute GPS position.

            @return an integer corresponding to the total count of satellites used to compute GPS position

            On failure, throws an exception or returns YGps.SATCOUNT_INVALID.
            """
            return self._run(self._aio.get_satCount())

    if not _DYNAMIC_HELPERS:
        def get_satPerConst(self) -> int:
            """
            Returns the count of visible satellites per constellation encoded
            on a 32 bit integer: bits 0..5: GPS satellites count,  bits 6..11 : Glonass, bits 12..17 : Galileo.
            this value is refreshed every 5 seconds only.

            @return an integer corresponding to the count of visible satellites per constellation encoded
                    on a 32 bit integer: bits 0.

            On failure, throws an exception or returns YGps.SATPERCONST_INVALID.
            """
            return self._run(self._aio.get_satPerConst())

    if not _DYNAMIC_HELPERS:
        def get_gpsRefreshRate(self) -> float:
            """
            Returns effective GPS data refresh frequency.
            this value is refreshed every 5 seconds only.

            @return a floating point number corresponding to effective GPS data refresh frequency

            On failure, throws an exception or returns YGps.GPSREFRESHRATE_INVALID.
            """
            return self._run(self._aio.get_gpsRefreshRate())

    if not _DYNAMIC_HELPERS:
        def get_coordSystem(self) -> int:
            """
            Returns the representation system used for positioning data.

            @return a value among YGps.COORDSYSTEM_GPS_DMS, YGps.COORDSYSTEM_GPS_DM and YGps.COORDSYSTEM_GPS_D
            corresponding to the representation system used for positioning data

            On failure, throws an exception or returns YGps.COORDSYSTEM_INVALID.
            """
            return self._run(self._aio.get_coordSystem())

    if not _DYNAMIC_HELPERS:
        def set_coordSystem(self, newval: int) -> int:
            """
            Changes the representation system used for positioning data.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a value among YGps.COORDSYSTEM_GPS_DMS, YGps.COORDSYSTEM_GPS_DM and
            YGps.COORDSYSTEM_GPS_D corresponding to the representation system used for positioning data

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_coordSystem(newval))

    if not _DYNAMIC_HELPERS:
        def get_constellation(self) -> int:
            """
            Returns the the satellites constellation used to compute
            positioning data.

            @return a value among YGps.CONSTELLATION_GNSS, YGps.CONSTELLATION_GPS, YGps.CONSTELLATION_GLONASS,
            YGps.CONSTELLATION_GALILEO, YGps.CONSTELLATION_GPS_GLONASS, YGps.CONSTELLATION_GPS_GALILEO and
            YGps.CONSTELLATION_GLONASS_GALILEO corresponding to the the satellites constellation used to compute
                    positioning data

            On failure, throws an exception or returns YGps.CONSTELLATION_INVALID.
            """
            return self._run(self._aio.get_constellation())

    if not _DYNAMIC_HELPERS:
        def set_constellation(self, newval: int) -> int:
            """
            Changes the satellites constellation used to compute
            positioning data. Possible  constellations are GNSS ( = all supported constellations),
            GPS, Glonass, Galileo , and the 3 possible pairs. This setting has  no effect on Yocto-GPS (V1).

            @param newval : a value among YGps.CONSTELLATION_GNSS, YGps.CONSTELLATION_GPS,
            YGps.CONSTELLATION_GLONASS, YGps.CONSTELLATION_GALILEO, YGps.CONSTELLATION_GPS_GLONASS,
            YGps.CONSTELLATION_GPS_GALILEO and YGps.CONSTELLATION_GLONASS_GALILEO corresponding to the
            satellites constellation used to compute
                    positioning data

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_constellation(newval))

    if not _DYNAMIC_HELPERS:
        def get_latitude(self) -> str:
            """
            Returns the current latitude.

            @return a string corresponding to the current latitude

            On failure, throws an exception or returns YGps.LATITUDE_INVALID.
            """
            return self._run(self._aio.get_latitude())

    if not _DYNAMIC_HELPERS:
        def get_longitude(self) -> str:
            """
            Returns the current longitude.

            @return a string corresponding to the current longitude

            On failure, throws an exception or returns YGps.LONGITUDE_INVALID.
            """
            return self._run(self._aio.get_longitude())

    if not _DYNAMIC_HELPERS:
        def get_dilution(self) -> float:
            """
            Returns the current horizontal dilution of precision,
            the smaller that number is, the better .

            @return a floating point number corresponding to the current horizontal dilution of precision,
                    the smaller that number is, the better

            On failure, throws an exception or returns YGps.DILUTION_INVALID.
            """
            return self._run(self._aio.get_dilution())

    if not _DYNAMIC_HELPERS:
        def get_altitude(self) -> float:
            """
            Returns the current altitude. Beware:  GPS technology
            is very inaccurate regarding altitude.

            @return a floating point number corresponding to the current altitude

            On failure, throws an exception or returns YGps.ALTITUDE_INVALID.
            """
            return self._run(self._aio.get_altitude())

    if not _DYNAMIC_HELPERS:
        def get_groundSpeed(self) -> float:
            """
            Returns the current ground speed in Km/h.

            @return a floating point number corresponding to the current ground speed in Km/h

            On failure, throws an exception or returns YGps.GROUNDSPEED_INVALID.
            """
            return self._run(self._aio.get_groundSpeed())

    if not _DYNAMIC_HELPERS:
        def get_direction(self) -> float:
            """
            Returns the current move bearing in degrees, zero
            is the true (geographic) north.

            @return a floating point number corresponding to the current move bearing in degrees, zero
                    is the true (geographic) north

            On failure, throws an exception or returns YGps.DIRECTION_INVALID.
            """
            return self._run(self._aio.get_direction())

    if not _DYNAMIC_HELPERS:
        def get_unixTime(self) -> int:
            """
            Returns the current time in Unix format (number of
            seconds elapsed since Jan 1st, 1970).

            @return an integer corresponding to the current time in Unix format (number of
                    seconds elapsed since Jan 1st, 1970)

            On failure, throws an exception or returns YGps.UNIXTIME_INVALID.
            """
            return self._run(self._aio.get_unixTime())

    if not _DYNAMIC_HELPERS:
        def get_dateTime(self) -> str:
            """
            Returns the current time in the form "YYYY/MM/DD hh:mm:ss".

            @return a string corresponding to the current time in the form "YYYY/MM/DD hh:mm:ss"

            On failure, throws an exception or returns YGps.DATETIME_INVALID.
            """
            return self._run(self._aio.get_dateTime())

    if not _DYNAMIC_HELPERS:
        def get_utcOffset(self) -> int:
            """
            Returns the number of seconds between current time and UTC time (time zone).

            @return an integer corresponding to the number of seconds between current time and UTC time (time zone)

            On failure, throws an exception or returns YGps.UTCOFFSET_INVALID.
            """
            return self._run(self._aio.get_utcOffset())

    if not _DYNAMIC_HELPERS:
        def set_utcOffset(self, newval: int) -> int:
            """
            Changes the number of seconds between current time and UTC time (time zone).
            The timezone is automatically rounded to the nearest multiple of 15 minutes.
            If current UTC time is known, the current time is automatically be updated according to the selected time zone.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : an integer corresponding to the number of seconds between current time and UTC time (time zone)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_utcOffset(newval))

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindGps(cls, func: str) -> YGps:
        """
        Retrieves a geolocalization module for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the geolocalization module is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YGps.isOnline() to test if the geolocalization module is
        indeed online at a given time. In case of ambiguity when looking for
        a geolocalization module by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the geolocalization module, for instance
                YGNSSMK2.gps.

        @return a YGps object allowing you to drive the geolocalization module.
        """
        return cls._proxy(cls, YGps_aio.FindGps(func))

    @classmethod
    def FindGpsInContext(cls, yctx: YAPIContext, func: str) -> YGps:
        """
        Retrieves a geolocalization module for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the geolocalization module is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YGps.isOnline() to test if the geolocalization module is
        indeed online at a given time. In case of ambiguity when looking for
        a geolocalization module by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the geolocalization module, for instance
                YGNSSMK2.gps.

        @return a YGps object allowing you to drive the geolocalization module.
        """
        return cls._proxy(cls, YGps_aio.FindGpsInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YGpsValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    # --- (end of YGps implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YTemperature
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YTemperature
version: PATCH_WITH_VERSION
requires: yocto_temperature_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_temperature_aio import YTemperature as YTemperature_aio
from .yocto_api import (
    YAPIContext, YAPI, YSensor, YMeasure
)

# --- (YTemperature class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YTemperatureValueCallback = Union[Callable[['YTemperature', str], Awaitable[None]], None]
        YTemperatureTimedReportCallback = Union[Callable[['YTemperature', YMeasure], Awaitable[None]], None]
    except TypeError:
        YTemperatureValueCallback = Union[Callable, Awaitable]
        YTemperatureTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YTemperature(YSensor):
    """
    The YTemperature class allows you to read and configure Yoctopuce temperature sensors.
    It inherits from YSensor class the core functions to read measures,
    to register callback functions, and to access the autonomous datalogger.
    This class adds the ability to configure some specific parameters
    for some sensors (connection type, temperature mapping table).

    """
    _aio: YTemperature_aio
    # --- (end of YTemperature class start)
    if not _IS_MICROPYTHON:
        # --- (YTemperature return codes)
        SIGNALVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        SIGNALUNIT_INVALID: Final[str] = YAPI.INVALID_STRING
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        SENSORTYPE_DIGITAL: Final[int] = 0
        SENSORTYPE_TYPE_K: Final[int] = 1
        SENSORTYPE_TYPE_E: Final[int] = 2
        SENSORTYPE_TYPE_J: Final[int] = 3
        SENSORTYPE_TYPE_N: Final[int] = 4
        SENSORTYPE_TYPE_R: Final[int] = 5
        SENSORTYPE_TYPE_S: Final[int] = 6
        SENSORTYPE_TYPE_T: Final[int] = 7
        SENSORTYPE_PT100_4WIRES: Final[int] = 8
        SENSORTYPE_PT100_3WIRES: Final[int] = 9
        SENSORTYPE_PT100_2WIRES: Final[int] = 10
        SENSORTYPE_RES_OHM: Final[int] = 11
        SENSORTYPE_RES_NTC: Final[int] = 12
        SENSORTYPE_RES_LINEAR: Final[int] = 13
        SENSORTYPE_RES_INTERNAL: Final[int] = 14
        SENSORTYPE_IR: Final[int] = 15
        SENSORTYPE_RES_PT1000: Final[int] = 16
        SENSORTYPE_CHANNEL_OFF: Final[int] = 17
        SENSORTYPE_INVALID: Final[int] = -1
        # --- (end of YTemperature return codes)


    # --- (YTemperature implementation)

    @classmethod
    def FirstTemperature(cls) -> Union[YTemperature, None]:
        """
        Starts the enumeration of temperature sensors currently accessible.
        Use the method YTemperature.nextTemperature() to iterate on
        next temperature sensors.

        @return a pointer to a YTemperature object, corresponding to
                the first temperature sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YTemperature_aio.FirstTemperature())

    @classmethod
    def FirstTemperatureInContext(cls, yctx: YAPIContext) -> Union[YTemperature, None]:
        """
        Starts the enumeration of temperature sensors currently accessible.
        Use the method YTemperature.nextTemperature() to iterate on
        next temperature sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YTemperature object, corresponding to
                the first temperature sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YTemperature_aio.FirstTemperatureInContext(yctx))

    def nextTemperature(self):
        """
        Continues the enumeration of temperature sensors started using yFirstTemperature().
        Caution: You can't make any assumption about the returned temperature sensors order.
        If you want to find a specific a temperature sensor, use Temperature.findTemperature()
        and a hardwareID or a logical name.

        @return a pointer to a YTemperature object, corresponding to
                a temperature sensor currently online, or a None pointer
                if there are no more temperature sensors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextTemperature())

    if not _DYNAMIC_HELPERS:
        def set_unit(self, newval: str) -> int:
            """
            Changes the measuring unit for the measured temperature. That unit is a string.
            If that strings end with the letter F all temperatures values will returned in
            Fahrenheit degrees. If that String ends with the letter K all values will be
            returned in Kelvin degrees. If that string ends with the letter C all values will be
            returned in Celsius degrees.  If the string ends with any other character the
            change will be ignored. Remember to call the
            saveToFlash() method of the module if the modification must be kept.
            WARNING: if a specific calibration is defined for the temperature function, a
            unit system change will probably break it.

            @param newval : a string corresponding to the measuring unit for the measured temperature

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_unit(newval))

    if not _DYNAMIC_HELPERS:
        def get_sensorType(self) -> int:
            """
            Returns the temperature sensor type.

            @return a value among YTemperature.SENSORTYPE_DIGITAL, YTemperature.SENSORTYPE_TYPE_K,
            YTemperature.SENSORTYPE_TYPE_E, YTemperature.SENSORTYPE_TYPE_J, YTemperature.SENSORTYPE_TYPE_N,
            YTemperature.SENSORTYPE_TYPE_R, YTemperature.SENSORTYPE_TYPE_S, YTemperature.SENSORTYPE_TYPE_T,
            YTemperature.SENSORTYPE_PT100_4WIRES, YTemperature.SENSORTYPE_PT100_3WIRES,
            YTemperature.SENSORTYPE_PT100_2WIRES, YTemperature.SENSORTYPE_RES_OHM,
            YTemperature.SENSORTYPE_RES_NTC, YTemperature.SENSORTYPE_RES_LINEAR,
            YTemperature.SENSORTYPE_RES_INTERNAL, YTemperature.SENSORTYPE_IR,
            YTemperature.SENSORTYPE_RES_PT1000 and YTemperature.SENSORTYPE_CHANNEL_OFF corresponding to the
            temperature sensor type

            On failure, throws an exception or returns YTemperature.SENSORTYPE_INVALID.
            """
            return self._run(self._aio.get_sensorType())

    if not _DYNAMIC_HELPERS:
        def set_sensorType(self, newval: int) -> int:
            """
            Changes the temperature sensor type.  This function is used
            to define the type of thermocouple (K,E...) used with the device.
            It has no effect if module is using a digital sensor or a thermistor.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a value among YTemperature.SENSORTYPE_DIGITAL, YTemperature.SENSORTYPE_TYPE_K,
            YTemperature.SENSORTYPE_TYPE_E, YTemperature.SENSORTYPE_TYPE_J, YTemperature.SENSORTYPE_TYPE_N,
            YTemperature.SENSORTYPE_TYPE_R, YTemperature.SENSORTYPE_TYPE_S, YTemperature.SENSORTYPE_TYPE_T,
            YTemperature.SENSORTYPE_PT100_4WIRES, YTemperature.SENSORTYPE_PT100_3WIRES,
            YTemperature.SENSORTYPE_PT100_2WIRES, YTemperature.SENSORTYPE_RES_OHM,
            YTemperature.SENSORTYPE_RES_NTC, YTemperature.SENSORTYPE_RES_LINEAR,
            YTemperature.SENSORTYPE_RES_INTERNAL, YTemperature.SENSORTYPE_IR,
            YTemperature.SENSORTYPE_RES_PT1000 and YTemperature.SENSORTYPE_CHANNEL_OFF corresponding to the
            temperature sensor type

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_sensorType(newval))

    if not _DYNAMIC_HELPERS:
        def get_signalValue(self) -> float:
            """
            Returns the current value of the electrical signal measured by the sensor.

            @return a floating point number corresponding to the current value of the electrical signal
            measured by the sensor

            On failure, throws an exception or returns YTemperature.SIGNALVALUE_INVALID.
            """
            return self._run(self._aio.get_signalValue())

    if not _DYNAMIC_HELPERS:
        def get_signalUnit(self) -> str:
            """
            Returns the measuring unit of the electrical signal used by the sensor.

            @return a string corresponding to the measuring unit of the electrical signal used by the sensor

            On failure, throws an exception or returns YTemperature.SIGNALUNIT_INVALID.
            """
            return self._run(self._aio.get_signalUnit())

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindTemperature(cls, func: str) -> YTemperature:
        """
        Retrieves a temperature sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the temperature sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YTemperature.isOnline() to test if the temperature sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a temperature sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the temperature sensor, for instance
                METEOMK2.temperature.

        @return a YTemperature object allowing you to drive the temperature sensor.
        """
        return cls._proxy(cls, YTemperature_aio.FindTemperature(func))

    @classmethod
    def FindTemperatureInContext(cls, yctx: YAPIContext, func: str) -> YTemperature:
        """
        Retrieves a temperature sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the temperature sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YTemperature.isOnline() to test if the temperature sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a temperature sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the temperature sensor, for instance
                METEOMK2.temperature.

        @return a YTemperature object allowing you to drive the temperature sensor.
        """
        return cls._proxy(cls, YTemperature_aio.FindTemperatureInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YTemperatureValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YTemperatureTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    if not _DYNAMIC_HELPERS:
        def set_ntcParameters(self, res25: float, beta: float) -> int:
            """
            Configures NTC thermistor parameters in order to properly compute the temperature from
            the measured resistance. For increased precision, you can enter a complete mapping
            table using set_thermistorResponseTable. This function can only be used with a
            temperature sensor based on thermistors.

            @param res25 : thermistor resistance at 25 degrees Celsius
            @param beta : Beta value

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_ntcParameters(res25, beta))

    if not _DYNAMIC_HELPERS:
        def set_thermistorResponseTable(self, tempValues: list[float], resValues: list[float]) -> int:
            """
            Records a thermistor response table, in order to interpolate the temperature from
            the measured resistance. This function can only be used with a temperature
            sensor based on thermistors.

            @param tempValues : array of floating point numbers, corresponding to all
                    temperatures (in degrees Celsius) for which the resistance of the
                    thermistor is specified.
            @param resValues : array of floating point numbers, corresponding to the resistance
                    values (in Ohms) for each of the temperature included in the first
                    argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_thermistorResponseTable(tempValues, resValues))

    if not _DYNAMIC_HELPERS:
        def loadThermistorResponseTable(self, tempValues: list[float], resValues: list[float]) -> int:
            """
            Retrieves the thermistor response table previously configured using the
            set_thermistorResponseTable function. This function can only be used with a
            temperature sensor based on thermistors.

            @param tempValues : array of floating point numbers, that is filled by the function
                    with all temperatures (in degrees Celsius) for which the resistance
                    of the thermistor is specified.
            @param resValues : array of floating point numbers, that is filled by the function
                    with the value (in Ohms) for each of the temperature included in the
                    first argument, index by index.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.loadThermistorResponseTable(tempValues, resValues))

    # --- (end of YTemperature implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YPowerSupply
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YPowerSupply
version: PATCH_WITH_VERSION
requires: yocto_powersupply_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_powersupply_aio import YPowerSupply as YPowerSupply_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YPowerSupply class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YPowerSupplyValueCallback = Union[Callable[['YPowerSupply', str], Awaitable[None]], None]
    except TypeError:
        YPowerSupplyValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YPowerSupply(YFunction):
    """
    The YPowerSupply class allows you to drive a Yoctopuce power supply.
    It can be use to change the voltage and current limits, and to enable/disable
    the output.

    """
    _aio: YPowerSupply_aio
    # --- (end of YPowerSupply class start)
    if not _IS_MICROPYTHON:
        # --- (YPowerSupply return codes)
        VOLTAGELIMIT_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        CURRENTLIMIT_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        MEASUREDVOLTAGE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        MEASUREDCURRENT_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        INPUTVOLTAGE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        VOLTAGETRANSITION_INVALID: Final[str] = YAPI.INVALID_STRING
        VOLTAGELIMITATSTARTUP_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        CURRENTLIMITATSTARTUP_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        POWEROUTPUT_OFF: Final[int] = 0
        POWEROUTPUT_ON: Final[int] = 1
        POWEROUTPUT_INVALID: Final[int] = -1
        POWEROUTPUTATSTARTUP_OFF: Final[int] = 0
        POWEROUTPUTATSTARTUP_ON: Final[int] = 1
        POWEROUTPUTATSTARTUP_INVALID: Final[int] = -1
        # --- (end of YPowerSupply return codes)


    # --- (YPowerSupply implementation)

    @classmethod
    def FirstPowerSupply(cls) -> Union[YPowerSupply, None]:
        """
        Starts the enumeration of regulated power supplies currently accessible.
        Use the method YPowerSupply.nextPowerSupply() to iterate on
        next regulated power supplies.

        @return a pointer to a YPowerSupply object, corresponding to
                the first regulated power supply currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YPowerSupply_aio.FirstPowerSupply())

    @classmethod
    def FirstPowerSupplyInContext(cls, yctx: YAPIContext) -> Union[YPowerSupply, None]:
        """
        Starts the enumeration of regulated power supplies currently accessible.
        Use the method YPowerSupply.nextPowerSupply() to iterate on
        next regulated power supplies.

        @param yctx : a YAPI context.

        @return a pointer to a YPowerSupply object, corresponding to
                the first regulated power supply currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YPowerSupply_aio.FirstPowerSupplyInContext(yctx))

    def nextPowerSupply(self):
        """
        Continues the enumeration of regulated power supplies started using yFirstPowerSupply().
        Caution: You can't make any assumption about the returned regulated power supplies order.
        If you want to find a specific a regulated power supply, use PowerSupply.findPowerSupply()
        and a hardwareID or a logical name.

        @return a pointer to a YPowerSupply object, corresponding to
                a regulated power supply currently online, or a None pointer
                if there are no more regulated power supplies to enumerate.
        """
        return self._proxy(type(self), self._aio.nextPowerSupply())

    if not _DYNAMIC_HELPERS:
        def set_voltageLimit(self, newval: float) -> int:
            """
            Changes the voltage limit, in V.

            @param newval : a floating point number corresponding to the voltage limit, in V

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_voltageLimit(newval))

    if not _DYNAMIC_HELPERS:
        def get_voltageLimit(self) -> float:
            """
            Returns the voltage limit, in V.

            @return a floating point number corresponding to the voltage limit, in V

            On failure, throws an exception or returns YPowerSupply.VOLTAGELIMIT_INVALID.
            """
            return self._run(self._aio.get_voltageLimit())

    if not _DYNAMIC_HELPERS:
        def set_currentLimit(self, newval: float) -> int:
            """
            Changes the current limit, in mA.

            @param newval : a floating point number corresponding to the current limit, in mA

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_currentLimit(newval))

    if not _DYNAMIC_HELPERS:
        def get_currentLimit(self) -> float:
            """
            Returns the current limit, in mA.

            @return a floating point number corresponding to the current limit, in mA

            On failure, throws an exception or returns YPowerSupply.CURRENTLIMIT_INVALID.
            """
            return self._run(self._aio.get_currentLimit())

    if not _DYNAMIC_HELPERS:
        def get_powerOutput(self) -> int:
            """
            Returns the power supply output switch state.

            @return either YPowerSupply.POWEROUTPUT_OFF or YPowerSupply.POWEROUTPUT_ON, according to the power
            supply output switch state

            On failure, throws an exception or returns YPowerSupply.POWEROUTPUT_INVALID.
            """
            return self._run(self._aio.get_powerOutput())

    if not _DYNAMIC_HELPERS:
        def set_powerOutput(self, newval: int) -> int:
            """
            Changes the power supply output switch state.

            @param newval : either YPowerSupply.POWEROUTPUT_OFF or YPowerSupply.POWEROUTPUT_ON, according to
            the power supply output switch state

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_powerOutput(newval))

    if not _DYNAMIC_HELPERS:
        def get_measuredVoltage(self) -> float:
            """
            Returns the measured output voltage, in V.

            @return a floating point number corresponding to the measured output voltage, in V

            On failure, throws an exception or returns YPowerSupply.MEASUREDVOLTAGE_INVALID.
            """
            return self._run(self._aio.get_measuredVoltage())

    if not _DYNAMIC_HELPERS:
        def get_measuredCurrent(self) -> float:
            """
            Returns the measured output current, in mA.

            @return a floating point number corresponding to the measured output current, in mA

            On failure, throws an exception or returns YPowerSupply.MEASUREDCURRENT_INVALID.
            """
            return self._run(self._aio.get_measuredCurrent())

    if not _DYNAMIC_HELPERS:
        def get_inputVoltage(self) -> float:
            """
            Returns the measured input voltage, in V.

            @return a floating point number corresponding to the measured input voltage, in V

            On failure, throws an exception or returns YPowerSupply.INPUTVOLTAGE_INVALID.
            """
            return self._run(self._aio.get_inputVoltage())

    if not _DYNAMIC_HELPERS:
        def set_voltageTransition(self, newval: str) -> int:
            return self._run(self._aio.set_voltageTransition(newval))

    if not _DYNAMIC_HELPERS:
        def set_voltageLimitAtStartUp(self, newval: float) -> int:
            """
            Changes the voltage set point at device start up. Remember to call the matching
            module saveToFlash() method, otherwise this call has no effect.

            @param newval : a floating point number corresponding to the voltage set point at device start up

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_voltageLimitAtStartUp(newval))

    if not _DYNAMIC_HELPERS:
        def get_voltageLimitAtStartUp(self) -> float:
            """
            Returns the selected voltage limit at device startup, in V.

            @return a floating point number corresponding to the selected voltage limit at device startup, in V

            On failure, throws an exception or returns YPowerSupply.VOLTAGELIMITATSTARTUP_INVALID.
            """
            return self._run(self._aio.get_voltageLimitAtStartUp())

    if not _DYNAMIC_HELPERS:
        def set_currentLimitAtStartUp(self, newval: float) -> int:
            """
            Changes the current limit at device start up. Remember to call the matching
            module saveToFlash() method, otherwise this call has no effect.

            @param newval : a floating point number corresponding to the current limit at device start up

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_currentLimitAtStartUp(newval))

    if not _DYNAMIC_HELPERS:
        def get_currentLimitAtStartUp(self) -> float:
            """
            Returns the selected current limit at device startup, in mA.

            @return a floating point number corresponding to the selected current limit at device startup, in mA

            On failure, throws an exception or returns YPowerSupply.CURRENTLIMITATSTARTUP_INVALID.
            """
            return self._run(self._aio.get_currentLimitAtStartUp())

    if not _DYNAMIC_HELPERS:
        def get_powerOutputAtStartUp(self) -> int:
            """
            Returns the power supply output switch state.

            @return either YPowerSupply.POWEROUTPUTATSTARTUP_OFF or YPowerSupply.POWEROUTPUTATSTARTUP_ON,
            according to the power supply output switch state

            On failure, throws an exception or returns YPowerSupply.POWEROUTPUTATSTARTUP_INVALID.
            """
            return self._run(self._aio.get_powerOutputAtStartUp())

    if not _DYNAMIC_HELPERS:
        def set_powerOutputAtStartUp(self, newval: int) -> int:
            """
            Changes the power supply output switch state at device start up. Remember to call the matching
            module saveToFlash() method, otherwise this call has no effect.

            @param newval : either YPowerSupply.POWEROUTPUTATSTARTUP_OFF or
            YPowerSupply.POWEROUTPUTATSTARTUP_ON, according to the power supply output switch state at device start up

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_powerOutputAtStartUp(newval))

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindPowerSupply(cls, func: str) -> YPowerSupply:
        """
        Retrieves a regulated power supply for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the regulated power supply is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPowerSupply.isOnline() to test if the regulated power supply is
        indeed online at a given time. In case of ambiguity when looking for
        a regulated power supply by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the regulated power supply, for instance
                MyDevice.powerSupply.

        @return a YPowerSupply object allowing you to drive the regulated power supply.
        """
        return cls._proxy(cls, YPowerSupply_aio.FindPowerSupply(func))

    @classmethod
    def FindPowerSupplyInContext(cls, yctx: YAPIContext, func: str) -> YPowerSupply:
        """
        Retrieves a regulated power supply for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the regulated power supply is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPowerSupply.isOnline() to test if the regulated power supply is
        indeed online at a given time. In case of ambiguity when looking for
        a regulated power supply by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the regulated power supply, for instance
                MyDevice.powerSupply.

        @return a YPowerSupply object allowing you to drive the regulated power supply.
        """
        return cls._proxy(cls, YPowerSupply_aio.FindPowerSupplyInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YPowerSupplyValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def voltageMove(self, V_target: float, ms_duration: int) -> int:
            """
            Performs a smooth transition of output voltage. Any explicit voltage
            change cancels any ongoing transition process.

            @param V_target   : new output voltage value at the end of the transition
                    (floating-point number, representing the end voltage in V)
            @param ms_duration : total duration of the transition, in milliseconds

            @return YAPI.SUCCESS when the call succeeds.
            """
            return self._run(self._aio.voltageMove(V_target, ms_duration))

    # --- (end of YPowerSupply implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YCurrentLoopOutput
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YCurrentLoopOutput
version: PATCH_WITH_VERSION
requires: yocto_currentloopoutput_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_currentloopoutput_aio import YCurrentLoopOutput as YCurrentLoopOutput_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YCurrentLoopOutput class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YCurrentLoopOutputValueCallback = Union[Callable[['YCurrentLoopOutput', str], Awaitable[None]], None]
    except TypeError:
        YCurrentLoopOutputValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YCurrentLoopOutput(YFunction):
    """
    The YCurrentLoopOutput class allows you to drive a 4-20mA output
    by regulating the current flowing through the current loop.
    It can also provide information about the power state of the current loop.

    """
    _aio: YCurrentLoopOutput_aio
    # --- (end of YCurrentLoopOutput class start)
    if not _IS_MICROPYTHON:
        # --- (YCurrentLoopOutput return codes)
        CURRENT_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        CURRENTTRANSITION_INVALID: Final[str] = YAPI.INVALID_STRING
        CURRENTATSTARTUP_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        LOOPPOWER_NOPWR: Final[int] = 0
        LOOPPOWER_LOWPWR: Final[int] = 1
        LOOPPOWER_POWEROK: Final[int] = 2
        LOOPPOWER_INVALID: Final[int] = -1
        # --- (end of YCurrentLoopOutput return codes)


    # --- (YCurrentLoopOutput implementation)

    @classmethod
    def FirstCurrentLoopOutput(cls) -> Union[YCurrentLoopOutput, None]:
        """
        Starts the enumeration of 4-20mA outputs currently accessible.
        Use the method YCurrentLoopOutput.nextCurrentLoopOutput() to iterate on
        next 4-20mA outputs.

        @return a pointer to a YCurrentLoopOutput object, corresponding to
                the first 4-20mA output currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YCurrentLoopOutput_aio.FirstCurrentLoopOutput())

    @classmethod
    def FirstCurrentLoopOutputInContext(cls, yctx: YAPIContext) -> Union[YCurrentLoopOutput, None]:
        """
        Starts the enumeration of 4-20mA outputs currently accessible.
        Use the method YCurrentLoopOutput.nextCurrentLoopOutput() to iterate on
        next 4-20mA outputs.

        @param yctx : a YAPI context.

        @return a pointer to a YCurrentLoopOutput object, corresponding to
                the first 4-20mA output currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YCurrentLoopOutput_aio.FirstCurrentLoopOutputInContext(yctx))

    def nextCurrentLoopOutput(self):
        """
        Continues the enumeration of 4-20mA outputs started using yFirstCurrentLoopOutput().
        Caution: You can't make any assumption about the returned 4-20mA outputs order.
        If you want to find a specific a 4-20mA output, use CurrentLoopOutput.findCurrentLoopOutput()
        and a hardwareID or a logical name.

        @return a pointer to a YCurrentLoopOutput object, corresponding to
                a 4-20mA output currently online, or a None pointer
                if there are no more 4-20mA outputs to enumerate.
        """
        return self._proxy(type(self), self._aio.nextCurrentLoopOutput())

    if not _DYNAMIC_HELPERS:
        def set_current(self, newval: float) -> int:
            """
            Changes the current loop, the valid range is from 3 to 21mA. If the loop is
            not properly powered, the  target current is not reached and
            loopPower is set to LOWPWR.

            @param newval : a floating point number corresponding to the current loop, the valid range is from 3 to 21mA

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_current(newval))

    if not _DYNAMIC_HELPERS:
        def get_current(self) -> float:
            """
            Returns the loop current set point in mA.

            @return a floating point number corresponding to the loop current set point in mA

            On failure, throws an exception or returns YCurrentLoopOutput.CURRENT_INVALID.
            """
            return self._run(self._aio.get_current())

    if not _DYNAMIC_HELPERS:
        def set_currentTransition(self, newval: str) -> int:
            return self._run(self._aio.set_currentTransition(newval))

    if not _DYNAMIC_HELPERS:
        def set_currentAtStartUp(self, newval: float) -> int:
            """
            Changes the loop current at device start up. Remember to call the matching
            module saveToFlash() method, otherwise this call has no effect.

            @param newval : a floating point number corresponding to the loop current at device start up

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_currentAtStartUp(newval))

    if not _DYNAMIC_HELPERS:
        def get_currentAtStartUp(self) -> float:
            """
            Returns the current in the loop at device startup, in mA.

            @return a floating point number corresponding to the current in the loop at device startup, in mA

            On failure, throws an exception or returns YCurrentLoopOutput.CURRENTATSTARTUP_INVALID.
            """
            return self._run(self._aio.get_currentAtStartUp())

    if not _DYNAMIC_HELPERS:
        def get_loopPower(self) -> int:
            """
            Returns the loop powerstate.  POWEROK: the loop
            is powered. NOPWR: the loop in not powered. LOWPWR: the loop is not
            powered enough to maintain the current required (insufficient voltage).

            @return a value among YCurrentLoopOutput.LOOPPOWER_NOPWR, YCurrentLoopOutput.LOOPPOWER_LOWPWR and
            YCurrentLoopOutput.LOOPPOWER_POWEROK corresponding to the loop powerstate

            On failure, throws an exception or returns YCurrentLoopOutput.LOOPPOWER_INVALID.
            """
            return self._run(self._aio.get_loopPower())

    @classmethod
    def FindCurrentLoopOutput(cls, func: str) -> YCurrentLoopOutput:
        """
        Retrieves a 4-20mA output for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the 4-20mA output is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YCurrentLoopOutput.isOnline() to test if the 4-20mA output is
        indeed online at a given time. In case of ambiguity when looking for
        a 4-20mA output by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the 4-20mA output, for instance
                TX420MA1.currentLoopOutput.

        @return a YCurrentLoopOutput object allowing you to drive the 4-20mA output.
        """
        return cls._proxy(cls, YCurrentLoopOutput_aio.FindCurrentLoopOutput(func))

    @classmethod
    def FindCurrentLoopOutputInContext(cls, yctx: YAPIContext, func: str) -> YCurrentLoopOutput:
        """
        Retrieves a 4-20mA output for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the 4-20mA output is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YCurrentLoopOutput.isOnline() to test if the 4-20mA output is
        indeed online at a given time. In case of ambiguity when looking for
        a 4-20mA output by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the 4-20mA output, for instance
                TX420MA1.currentLoopOutput.

        @return a YCurrentLoopOutput object allowing you to drive the 4-20mA output.
        """
        return cls._proxy(cls, YCurrentLoopOutput_aio.FindCurrentLoopOutputInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YCurrentLoopOutputValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def currentMove(self, mA_target: float, ms_duration: int) -> int:
            """
            Performs a smooth transition of current flowing in the loop. Any current explicit
            change cancels any ongoing transition process.

            @param mA_target   : new current value at the end of the transition
                    (floating-point number, representing the end current in mA)
            @param ms_duration : total duration of the transition, in milliseconds

            @return YAPI.SUCCESS when the call succeeds.
            """
            return self._run(self._aio.currentMove(mA_target, ms_duration))

    # --- (end of YCurrentLoopOutput implementation)




#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys

from yocto_api import *


def logfun(m, s):
    print(m.get_serialNumber() + ' : ' + s)


def deviceArrival(m):
    serial = m.get_serialNumber()
    print('Device arrival : ' + serial)
    m.registerLogCallback(logfun)


def deviceRemoval(m):
    print('Device removal : ' + m.get_serialNumber())


errmsg = YRefParam()

# No exception please
YAPI.DisableExceptions()

# Setup the API to use local USB devices
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

YAPI.RegisterDeviceArrivalCallback(deviceArrival)
YAPI.RegisterDeviceRemovalCallback(deviceRemoval)

print('Hit Ctrl-C to Stop ')
while True:
    YAPI.UpdateDeviceList(errmsg)  # traps plug/unplug events
    YAPI.Sleep(500, errmsg)  # traps others events



# ********************************************************************
#
#  $Id: helloworld.py 32630 2018-10-10 14:11:07Z seb $
#
#  An example that show how to use a  Yocto-Maxi-IO
#
#  You can find more information on our web site:
#   Yocto-Maxi-IO documentation:
#      https://www.yoctopuce.com/EN/products/yocto-maxi-io/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
#
# *********************************************************************

#!/usr/bin/python
# -*- coding: utf-8 -*-
import os, sys
# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))

from yocto_api import *
from yocto_digitalio import *


def usage():
    scriptname = os.path.basename(sys.argv[0])
    print("Usage:")
    print(scriptname + ' <serial_number>')
    print(scriptname + ' <logical_name>')
    print(scriptname + ' any')
    print('Example:')
    print(scriptname + ' any')
    sys.exit()


def die(msg):
    sys.exit(msg + ' (check USB cable)')


if len(sys.argv) < 2:
    usage()
target = sys.argv[1].upper()

# Setup the API to use local USB devices
errmsg = YRefParam()
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

if target == 'ANY':
    # retreive any Relay then find its serial #
    io = YDigitalIO.FirstDigitalIO()
    if io is None:
        die('No module connected')
    m = io.get_module()
    target = m.get_serialNumber()

print('using ' + target)
io = YDigitalIO.FindDigitalIO(target + '.digitalIO')

if not (io.isOnline()):
    die('device not connected')

# lets configure the channels direction
# bits 0..3 as output
# bits 4..7 as input
io.set_portDirection(0x0F)
io.set_portPolarity(0)  # polarity set to regular
io.set_portOpenDrain(0)  # No open drain

print("Channels 0..3 are configured as outputs and channels 4..7")
print("are configured as inputs, you can connect some inputs to ")
print("ouputs and see what happens")

outputdata = 0
while io.isOnline():
    inputdata = io.get_portState()  # read port values
    line = ""  # display part state value as binary
    for i in range(0, 8):
        if (inputdata & (128 >> i)) > 0:
            line += '1'
        else:
            line += '0'
    print(" port value = " + line)
    outputdata = (outputdata + 1) % 16  # cycle ouput 0..15
    io.set_portState(outputdata)  # We could have used set_bitState as well
    YAPI.Sleep(1000, errmsg)

print("Module disconnected")
YAPI.FreeAPI()
